@@ -4,11 +4,11 @@ void insert_model_tree(struct Node *tree, struct Var *vars,  int n, int *model,
 
 int *GetModel_m(SEXP Rmodel_m, int *model, int p) {
 	int *model_m = INTEGER(Rmodel_m);
-	for (int j = 0, l=0; j < p; j++) {  
+	for (int j = 0, l=0; j < p; j++) {
 		if (model[j] == 1) {
 			model_m[l++] = j;
 		}
-	}	
+	}
 	return model_m;
 }
 
@@ -16,7 +16,7 @@ void CreateTree(NODEPTR branch, struct Var *vars, int *bestmodel, int *model, in
 	for (int i = 0; i< n; i++) {
 		int bit =  bestmodel[vars[i].index];
 		if (bit == 1) {
-			if (i < n-1 && branch->one == NULL) 
+			if (i < n-1 && branch->one == NULL)
 				branch->one = make_node(-1.0);
 			if (i == n-1 && branch->one == NULL)
 				branch->one = make_node(0.0);
@@ -27,8 +27,8 @@ void CreateTree(NODEPTR branch, struct Var *vars, int *bestmodel, int *model, in
 			if (i == n-1 && branch->zero == NULL)
 				branch->zero = make_node(0.0);
 			branch = branch->zero;
-		} 
-		model[vars[i].index] = bit; 
+		}
+		model[vars[i].index] = bit;
 		INTEGER(modeldim)[m]  += bit;
 		branch->where = 0;
 	}
@@ -38,7 +38,7 @@ double FitModel(SEXP Rcoef_m, SEXP Rse_m, double *XtY, double *XtX, int *model_m
 			  double *XtYwork, double *XtXwork, double yty, double SSY, int pmodel, int p,
 			  int nobs, int m, double *pmse_m) {
 
-	double *coefficients = REAL(Rcoef_m);  
+	double *coefficients = REAL(Rcoef_m);
 	double *se_m = REAL(Rse_m);
 
 	for (int j=0; j < pmodel; j++) { //subsetting matrix
@@ -46,14 +46,14 @@ double FitModel(SEXP Rcoef_m, SEXP Rse_m, double *XtY, double *XtX, int *model_m
 		for  ( int i = 0; i < pmodel; i++) {
 			XtXwork[j*pmodel + i] = XtX[model_m[j]*p + model_m[i]];
 		}
-	} 
-	*pmse_m = yty; 
-	memcpy(coefficients, XtYwork, sizeof(double)*pmodel); 
-	cholreg(XtYwork, XtXwork, coefficients, se_m, pmse_m, pmodel, nobs);  
+	}
+	*pmse_m = yty;
+	memcpy(coefficients, XtYwork, sizeof(double)*pmodel);
+	cholreg(XtYwork, XtXwork, coefficients, se_m, pmse_m, pmodel, nobs);
 
 	double R2_m = 1.0 - (*pmse_m * (double) ( nobs - pmodel))/SSY;
         if (R2_m < 0.0 || pmodel == 1) R2_m = 0.0;
-	
+
 	return R2_m;
 }
 
@@ -66,16 +66,28 @@ void SetModel2(double logmargy, double shrinkage_m, double prior_m,
 }
 
 void SetModel(SEXP Rcoef_m, SEXP Rse_m, SEXP Rmodel_m, double mse_m, double R2_m,
-			  SEXP beta, SEXP se, SEXP modelspace, SEXP mse, SEXP R2, int m) {
+              SEXP beta, SEXP se, SEXP modelspace, SEXP mse, SEXP R2, int m) {
 
-	SET_ELEMENT(beta, m, Rcoef_m);
-	SET_ELEMENT(se, m, Rse_m);
-	SET_ELEMENT(modelspace, m, Rmodel_m);
+  SET_ELEMENT(beta, m, Rcoef_m);
+  SET_ELEMENT(se, m, Rse_m);
+  SET_ELEMENT(modelspace, m, Rmodel_m);
 
-	REAL(R2)[m] = R2_m;
-	REAL(mse)[m] = mse_m;
+  REAL(R2)[m] = R2_m;
+  REAL(mse)[m] = mse_m;
+
+  UNPROTECT(3);
+}
+
+void SetModel_lm(SEXP Rcoef_m, SEXP Rse_m, SEXP Rmodel_m, double mse_m, double R2_m,
+              SEXP beta, SEXP se, SEXP modelspace, SEXP mse, SEXP R2, int m) {
+
+  SET_ELEMENT(beta, m, Rcoef_m);
+  SET_ELEMENT(se, m, Rse_m);
+  SET_ELEMENT(modelspace, m, Rmodel_m);
+
+  REAL(R2)[m] = R2_m;
+  REAL(mse)[m] = mse_m;
 
-	UNPROTECT(3);
 }
 
 
@@ -98,8 +110,8 @@ double GetNextModelCandidate(int pmodel_old, int n, int n_sure, int *model, stru
 }
 
 // [[register]]
-extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim, SEXP incint, SEXP Ralpha,SEXP method, 
-			  SEXP modelprior, SEXP Rupdate, SEXP Rbestmodel,  SEXP plocal, 
+extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeldim, SEXP incint, SEXP Ralpha,SEXP method,
+			  SEXP modelprior, SEXP Rupdate, SEXP Rbestmodel,  SEXP plocal,
 	      SEXP BURNIN_Iterations, SEXP MCMC_Iterations, SEXP LAMBDA, SEXP DELTA, SEXP Rthin)
 {
 	int nProtected = 0;
@@ -127,12 +139,12 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 	SEXP NumUnique = PROTECT(allocVector(INTSXP, 1)); ++nProtected;
 
 	double *Xwork, *Ywork,*wts, *probs, shrinkage_m,
-		mse_m, MH=0.0, prior_m=1.0, 
+		mse_m, MH=0.0, prior_m=1.0,
 		R2_m, RSquareFull, logmargy, postold, postnew;
 	int i, m, n, pmodel_old, *model_m, *bestmodel;
 	int mcurrent, n_sure;
-	
-	//get dimsensions of all variables 
+
+	//get dimsensions of all variables
 	int nobs = LENGTH(Y);
 	int p = INTEGER(getAttrib(X,R_DimSymbol))[1];
 	int k = LENGTH(modelprobs);
@@ -140,28 +152,28 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 	//	double delta = REAL(DELTA)[0];
 	double alpha = REAL(Ralpha)[0];
 	int thin = INTEGER(Rthin)[0];
-	
+
 	//	Rprintf("delta %f lambda %f", delta, lambda);
 
 	Ywork = REAL(RYwork);
 	Xwork = REAL(RXwork);
-	wts = REAL(Rweights); 
-	
+	wts = REAL(Rweights);
+
 
 	double *XtXwork, *XtYwork,*XtX, *XtY, yty,SSY;
 	PrecomputeData(Xwork, Ywork, wts, &XtXwork, &XtYwork, &XtX, &XtY, &yty, &SSY, p, nobs);
 
-	
-	struct Var *vars = (struct Var *) R_alloc(p, sizeof(struct Var)); // Info about the model variables. 
+
+	struct Var *vars = (struct Var *) R_alloc(p, sizeof(struct Var)); // Info about the model variables.
 	probs =  REAL(Rprobs);
-	n = sortvars(vars, probs, p); 
+	n = sortvars(vars, probs, p);
 	for (i =n; i <p; i++) REAL(MCMCprobs)[vars[i].index] = probs[vars[i].index];
 	for (i =0; i <n; i++) REAL(MCMCprobs)[vars[i].index] = 0.0;
-	
-	SEXP Rse_m = NULL, Rcoef_m = NULL; 
+
+	SEXP Rse_m = NULL, Rcoef_m = NULL, Rmodel_m=NULL;
 	RSquareFull = CalculateRSquareFull(XtY, XtX, XtXwork, XtYwork, Rcoef_m, Rse_m, p, nobs, yty, SSY);
-	
-	// fill in the sure things 
+
+	// fill in the sure things
 	int *model = ivecalloc(p);
 	for (i = n, n_sure = 0; i < p; i++)  {
 		model[vars[i].index] = (int) vars[i].prob;
@@ -172,34 +184,33 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	NODEPTR tree, branch;
 	tree = make_node(-1.0);
-	//  Rprintf("For m=0, Initialize Tree with initial Model\n");  
+	//  Rprintf("For m=0, Initialize Tree with initial Model\n");
 
 	m = 0;
 	bestmodel = INTEGER(Rbestmodel);
 	INTEGER(modeldim)[m] = n_sure;
 
-	// Rprintf("Create Tree\n"); 
+	// Rprintf("Create Tree\n");
 	branch = tree;
 	CreateTree(branch, vars, bestmodel, model, n, m, modeldim);
 
 	int pmodel = INTEGER(modeldim)[m];
-	SEXP Rmodel_m;
 	PROTECT(Rmodel_m = allocVector(INTSXP,pmodel));
-	PROTECT(Rcoef_m = NEW_NUMERIC(pmodel)); 
-	PROTECT(Rse_m = NEW_NUMERIC(pmodel));  
+	PROTECT(Rcoef_m = NEW_NUMERIC(pmodel));
+	PROTECT(Rse_m = NEW_NUMERIC(pmodel));
 
 	model_m = GetModel_m(Rmodel_m, model, p);
 	//evaluate logmargy and shrinkage
 
 	R2_m = FitModel(Rcoef_m, Rse_m, XtY, XtX, model_m, XtYwork, XtXwork, yty, SSY, pmodel, p, nobs, m, &mse_m);
 	gexpectations(p, pmodel, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
-	
+
 	prior_m  = compute_prior_probs(model,pmodel,p, modelprior);
 	if (prior_m == 0.0)  Rprintf("warning initial model has 0 prior probabilty\n");
 	SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, m);
 	SetModel(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2, m);
 
-	int nUnique=0, newmodel=0, nsamples=0; 
+	int nUnique=0, newmodel=0, nsamples=0;
 	double *real_model = vecalloc(n);
 	int *modelold = ivecalloc(p);
 	int old_loc = 0;
@@ -215,11 +226,11 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 	double problocal = REAL(plocal)[0];
 	while (nUnique < k && m < INTEGER(BURNIN_Iterations)[0]) {
 
-	        memcpy(model, modelold, sizeof(int)*p);
+	  memcpy(model, modelold, sizeof(int)*p);
 		pmodel =  n_sure;
 
 		MH = GetNextModelCandidate(pmodel_old, n, n_sure, model, vars, problocal, varin, varout);
-		
+
 		branch = tree;
 		newmodel= 0;
 		for (i = 0; i< n; i++) {
@@ -230,7 +241,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 		  } else {
 		    if (branch->zero != NULL)  branch = branch->zero;
 		    else newmodel = 1;
-		  } 
+		  }
 		  pmodel  += bit;
 		}
 
@@ -246,14 +257,14 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 		  else {
 		    new_loc = nUnique;
 		    PROTECT(Rmodel_m = allocVector(INTSXP,pmodel));
-		    PROTECT(Rcoef_m = NEW_NUMERIC(pmodel)); 
-		    PROTECT(Rse_m = NEW_NUMERIC(pmodel));   
+		    PROTECT(Rcoef_m = NEW_NUMERIC(pmodel));
+		    PROTECT(Rse_m = NEW_NUMERIC(pmodel));
 		    model_m = GetModel_m(Rmodel_m, model, p);
 
 		    R2_m = FitModel(Rcoef_m, Rse_m, XtY, XtX, model_m, XtYwork, XtXwork, yty, SSY, pmodel, p, nobs, m, &mse_m);
 		    gexpectations(p, pmodel, nobs, R2_m, alpha, INTEGER(method)[0], RSquareFull, SSY, &logmargy, &shrinkage_m);
 
-		  
+
 		    postnew = logmargy + log(prior_m);
 		    MH *= exp(postnew - postold);
 		  }}
@@ -263,20 +274,20 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 		             log(REAL(priorprobs)[new_loc]);
 		  MH *=  exp(postnew - postold);
 		}
-		
+
 		//    Rprintf("MH new %lf old %lf\n", postnew, postold);
 		if (unif_rand() < MH) {
 		  if (newmodel == 1) {
 		    if ((m % thin) == 0 )  {
-		    
+
 		    new_loc = nUnique;
 		    insert_model_tree(tree, vars, n, model, nUnique);
 		    INTEGER(modeldim)[nUnique] = pmodel;
-		    
+
 		    //record model data
 		    SetModel2(logmargy, shrinkage_m, prior_m, sampleprobs, logmarg, shrinkage, priorprobs, nUnique);
 		    SetModel(Rcoef_m, Rse_m, Rmodel_m, mse_m, R2_m,	beta, se, modelspace, mse, R2,nUnique);
-		    
+
 		    ++nUnique;
 		    }
 		    else UNPROTECT(3);
@@ -296,26 +307,26 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 		  INTEGER(counts)[old_loc] += 1;
 
 		  for (i = 0; i < n; i++) {
-		    // store in opposite order so nth variable is first 
+		    // store in opposite order so nth variable is first
 		    real_model[n-1-i] = (double) modelold[vars[i].index];
-		    REAL(MCMCprobs)[vars[i].index] += (double) modelold[vars[i].index];	
+		    REAL(MCMCprobs)[vars[i].index] += (double) modelold[vars[i].index];
 		  }
-		  nsamples++; 
+		  nsamples++;
 		}
 		m++;
 	}
 
 	// Now wrap up
 
-	// Compute MCMC inclusion probabilities 
+	// Compute MCMC inclusion probabilities
 	for (i = 0; i < n; i++) {
 		REAL(MCMCprobs)[vars[i].index] /= (double) nsamples;
 	}
-	
-	// Compute marginal probabilities  
+
+	// Compute marginal probabilities
 	mcurrent = nUnique;
 	compute_modelprobs(modelprobs, logmarg, priorprobs,mcurrent);
-	compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);        
+	compute_margprobs(modelspace, modeldim, modelprobs, probs, mcurrent, p);
 
 	INTEGER(NumUnique)[0] = nUnique;
 
@@ -324,7 +335,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP modelspaceP = PROTECT(allocVector(VECSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			SEXP model_temp = PROTECT(VECTOR_ELT(modelspace, i));
 			SET_ELEMENT(modelspaceP, i, model_temp);
 			UNPROTECT(1);
@@ -338,7 +349,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP logmargP = PROTECT(allocVector(REALSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			REAL(logmargP)[i] = REAL(logmarg)[i];
 		}
 		SET_VECTOR_ELT(ANS, 2, logmargP);
@@ -350,7 +361,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP modelprobsP = PROTECT(allocVector(REALSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			REAL(modelprobsP)[i] = REAL(modelprobs)[i];
 		}
 		SET_VECTOR_ELT(ANS, 3, modelprobsP);
@@ -362,7 +373,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP priorprobsP = PROTECT(allocVector(REALSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			REAL(priorprobsP)[i] = REAL(priorprobs)[i];
 		}
 		SET_VECTOR_ELT(ANS, 4, priorprobsP);
@@ -374,7 +385,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP sampleprobsP = PROTECT(allocVector(REALSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			REAL(sampleprobsP)[i] = REAL(sampleprobs)[i];
 		}
 		SET_VECTOR_ELT(ANS, 5, sampleprobsP);
@@ -386,7 +397,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP mseP = PROTECT(allocVector(REALSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			REAL(mseP)[i] = REAL(mse)[i];
 		}
 		SET_VECTOR_ELT(ANS, 6, mseP);
@@ -398,7 +409,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP betaP = PROTECT(allocVector(VECSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			SEXP beta_temp = PROTECT(VECTOR_ELT(beta, i));
 			SET_ELEMENT(betaP, i, beta_temp);
 			UNPROTECT(1);
@@ -412,7 +423,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP seP = PROTECT(allocVector(VECSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			SEXP se_temp = PROTECT(VECTOR_ELT(se, i));
 			SET_ELEMENT(seP, i, se_temp);
 			UNPROTECT(1);
@@ -426,7 +437,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP shrinkageP = PROTECT(allocVector(REALSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			REAL(shrinkageP)[i] = REAL(shrinkage)[i];
 		}
 		SET_VECTOR_ELT(ANS, 9, shrinkageP);
@@ -438,7 +449,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP modeldimP = PROTECT(allocVector(INTSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			INTEGER(modeldimP)[i] = INTEGER(modeldim)[i];
 		}
 		SET_VECTOR_ELT(ANS, 10, modeldimP);
@@ -450,7 +461,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP R2P = PROTECT(allocVector(REALSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			REAL(R2P)[i] = REAL(R2)[i];
 		}
 		SET_VECTOR_ELT(ANS, 11, R2P);
@@ -462,7 +473,7 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 
 	if (nUnique < nModels) {
 		SEXP countsP = PROTECT(allocVector(INTSXP, nUnique));
-		for (i =0; i < nUnique; i++) { 
+		for (i =0; i < nUnique; i++) {
 			INTEGER(countsP)[i] = INTEGER(counts)[i];
 		}
 		SET_VECTOR_ELT(ANS, 12, countsP);
@@ -479,10 +490,10 @@ extern SEXP mcmc_new(SEXP Y, SEXP X, SEXP Rweights, SEXP Rprobinit, SEXP Rmodeld
 	SET_STRING_ELT(ANS_names, 14, mkChar("n.Unique"));
 
 	setAttrib(ANS, R_NamesSymbol, ANS_names);
-	
+
 	PutRNGstate();
     UNPROTECT(nProtected);
     //	Rprintf("Return\n");
-	return(ANS);  
+	return(ANS);
 }
 
